@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2009-2010 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2009-2012 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -110,22 +110,74 @@ gcm_calibrate_argyll_get_quality_arg (GcmCalibrateArgyll *calibrate_argyll)
 	return "-qm";
 }
 
+/**
+ * gcm_calibrate_argyll_get_tool_filename:
+ **/
+static gchar *
+gcm_calibrate_argyll_get_tool_filename (const gchar *command,
+					GError **error)
+{
+	gboolean ret;
+	gchar *filename;
+
+	/* try the original argyllcms filename installed in /usr/local/bin */
+	filename = g_strdup_printf ("/usr/local/bin/%s", command);
+	ret = g_file_test (filename, G_FILE_TEST_EXISTS);
+	if (ret)
+		goto out;
+
+	/* try the debian filename installed in /usr/bin */
+	g_free (filename);
+	filename = g_strdup_printf ("/usr/bin/argyll-%s", command);
+	ret = g_file_test (filename, G_FILE_TEST_EXISTS);
+	if (ret)
+		goto out;
+
+	/* try the original argyllcms filename installed in /usr/bin */
+	g_free (filename);
+	filename = g_strdup_printf ("/usr/bin/%s", command);
+	ret = g_file_test (filename, G_FILE_TEST_EXISTS);
+	if (ret)
+		goto out;
+
+	/* eek */
+	g_free (filename);
+	filename = NULL;
+	g_set_error (error,
+		     GCM_CALIBRATE_ERROR,
+		     GCM_CALIBRATE_ERROR_INTERNAL,
+		     "failed to get filename for %s", command);
+out:
+	return filename;
+}
+
 /**
  * gcm_calibrate_argyll_get_display:
  **/
 static guint
-gcm_calibrate_argyll_get_display (const gchar *output_name, GError **error)
+gcm_calibrate_argyll_get_display (const gchar *output_name,
+				  GError **error)
 {
-	gboolean ret;
+	gboolean ret = FALSE;
+	gchar *command = NULL;
 	gchar *data = NULL;
+	gchar *name;
 	gchar **split = NULL;
 	gint exit_status;
 	guint display = G_MAXUINT;
 	guint i;
-	gchar *name;
+
+	/* get correct name of the command */
+	command = gcm_calibrate_argyll_get_tool_filename ("dispcal", error);
+	if (command == NULL)
+		goto out;
 
 	/* execute it and capture stderr */
-	ret = g_spawn_command_line_sync ("dispcal", NULL, &data, &exit_status, error);
+	ret = g_spawn_command_line_sync (command,
+					 NULL,
+					 &data,
+					 &exit_status,
+					 error);
 	if (!ret)
 		goto out;
 
@@ -157,6 +209,7 @@ gcm_calibrate_argyll_get_display (const gchar *output_name, GError **error)
 		goto out;
 	}
 out:
+	g_free (command);
 	g_free (data);
 	g_strfreev (split);
 	return display;
@@ -195,51 +248,13 @@ gcm_calibrate_argyll_debug_argv (const gchar *program, gchar **argv)
 	g_free (join);
 }
 
-/**
- * gcm_calibrate_argyll_get_tool_filename:
- **/
-static gchar *
-gcm_calibrate_argyll_get_tool_filename (const gchar *command, GError **error)
-{
-	gboolean ret;
-	gchar *filename;
-
-	/* try the original argyllcms filename installed in /usr/local/bin */
-	filename = g_strdup_printf ("/usr/local/bin/%s", command);
-	ret = g_file_test (filename, G_FILE_TEST_EXISTS);
-	if (ret)
-		goto out;
-
-	/* try the debian filename installed in /usr/bin */
-	g_free (filename);
-	filename = g_strdup_printf ("/usr/bin/argyll-%s", command);
-	ret = g_file_test (filename, G_FILE_TEST_EXISTS);
-	if (ret)
-		goto out;
-
-	/* try the original argyllcms filename installed in /usr/bin */
-	g_free (filename);
-	filename = g_strdup_printf ("/usr/bin/%s", command);
-	ret = g_file_test (filename, G_FILE_TEST_EXISTS);
-	if (ret)
-		goto out;
-
-	/* eek */
-	g_free (filename);
-	filename = NULL;
-	g_set_error (error,
-		     GCM_CALIBRATE_ERROR,
-		     GCM_CALIBRATE_ERROR_INTERNAL,
-		     "failed to get filename for %s", command);
-out:
-	return filename;
-}
-
 /**
  * gcm_calibrate_argyll_fork_command:
  **/
 static gboolean
-gcm_calibrate_argyll_fork_command (GcmCalibrateArgyll *calibrate_argyll, gchar **argv, GError **error)
+gcm_calibrate_argyll_fork_command (GcmCalibrateArgyll *calibrate_argyll,
+				   gchar **argv,
+				   GError **error)
 {
 	gboolean ret = FALSE;
 #ifdef HAVE_VTE
@@ -283,7 +298,8 @@ out:
  * gcm_calibrate_argyll_display_neutralise:
  **/
 static gboolean
-gcm_calibrate_argyll_display_neutralise (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+gcm_calibrate_argyll_display_neutralise (GcmCalibrateArgyll *calibrate_argyll,
+					 GError **error)
 {
 	gboolean ret = TRUE;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
@@ -323,11 +339,13 @@ gcm_calibrate_argyll_display_neutralise (GcmCalibrateArgyll *calibrate_argyll, G
 
 	/* TRANSLATORS: title, default paramters needed to calibrate_argyll */
 	gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-				 _("Getting default parameters"));
+				 _("Getting default parameters"),
+				 GCM_CALIBRATE_UI_STATUS);
 
 	/* TRANSLATORS: dialog message */
 	gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   _("This pre-calibrates the screen by sending colored and gray patches to your screen and measuring them with the hardware device."));
+				   _("This pre-calibrates the screen by sending colored and gray patches to your screen and measuring them with the hardware device."),
+				   GCM_CALIBRATE_UI_STATUS);
 
 	/* argument array */
 	array = g_ptr_array_new_with_free_func (g_free);
@@ -391,7 +409,8 @@ out:
  * gcm_calibrate_argyll_display_read_chart:
  **/
 static gboolean
-gcm_calibrate_argyll_display_read_chart (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+gcm_calibrate_argyll_display_read_chart (GcmCalibrateArgyll *calibrate_argyll,
+					 GError **error)
 {
 	gboolean ret = TRUE;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
@@ -414,10 +433,13 @@ gcm_calibrate_argyll_display_read_chart (GcmCalibrateArgyll *calibrate_argyll, G
 
 	/* TRANSLATORS: title, patches are specific colours used in calibration */
 	gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-				 _("Reading the patches"));
+				 _("Reading the patches"),
+				 GCM_CALIBRATE_UI_STATUS);
+
 	/* TRANSLATORS: dialog message */
 	gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   _("Reading the patches using the color measuring instrument."));
+				   _("Reading the patches using the color measuring instrument."),
+				   GCM_CALIBRATE_UI_STATUS);
 
 	/* argument array */
 	array = g_ptr_array_new_with_free_func (g_free);
@@ -476,7 +498,8 @@ out:
  * gcm_calibrate_argyll_display_draw_and_measure:
  **/
 static gboolean
-gcm_calibrate_argyll_display_draw_and_measure (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+gcm_calibrate_argyll_display_draw_and_measure (GcmCalibrateArgyll *calibrate_argyll,
+					       GError **error)
 {
 	gboolean ret = TRUE;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
@@ -514,10 +537,13 @@ gcm_calibrate_argyll_display_draw_and_measure (GcmCalibrateArgyll *calibrate_arg
 
 	/* TRANSLATORS: title, drawing means painting to the screen */
 	gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-				 _("Drawing the patches"));
+				 _("Drawing the patches"),
+				 GCM_CALIBRATE_UI_STATUS);
+
 	/* TRANSLATORS: dialog message */
 	gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   _("Drawing the generated patches to the screen, which will then be measured by the hardware device."));
+				   _("Drawing the generated patches to the screen, which will then be measured by the hardware device."),
+				   GCM_CALIBRATE_UI_STATUS);
 
 	/* argument array */
 	array = g_ptr_array_new_with_free_func (g_free);
@@ -579,7 +605,8 @@ out:
  * gcm_calibrate_argyll_display_generate_profile:
  **/
 static gboolean
-gcm_calibrate_argyll_display_generate_profile (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+gcm_calibrate_argyll_display_generate_profile (GcmCalibrateArgyll *calibrate_argyll,
+					       GError **error)
 {
 	gboolean ret = TRUE;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
@@ -610,10 +637,13 @@ gcm_calibrate_argyll_display_generate_profile (GcmCalibrateArgyll *calibrate_arg
 
 	/* TRANSLATORS: title, a profile is a ICC file */
 	gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-				 _("Generating the profile"));
+				 _("Generating the profile"),
+				 GCM_CALIBRATE_UI_STATUS);
+
 	/* TRANSLATORS: dialog message */
 	gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   _("Generating the ICC color profile that can be used with this screen."));
+				   _("Generating the ICC color profile that can be used with this screen."),
+				   GCM_CALIBRATE_UI_STATUS);
 
 	/* argument array */
 	array = g_ptr_array_new_with_free_func (g_free);
@@ -710,7 +740,9 @@ gcm_calibrate_argyll_reference_kind_to_filename (GcmCalibrateReferenceKind kind)
  * gcm_utils_mkdir_and_copy:
  **/
 static gboolean
-gcm_utils_mkdir_and_copy (GFile *source, GFile *destination, GError **error)
+gcm_utils_mkdir_and_copy (GFile *source,
+			  GFile *destination,
+			  GError **error)
 {
 	gboolean ret;
 	GFile *parent;
@@ -729,7 +761,13 @@ gcm_utils_mkdir_and_copy (GFile *source, GFile *destination, GError **error)
 	}
 
 	/* do the copy */
-	ret = g_file_copy (source, destination, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, error);
+	ret = g_file_copy (source,
+			   destination,
+			   G_FILE_COPY_OVERWRITE,
+			   NULL,
+			   NULL,
+			   NULL,
+			   error);
 	if (!ret)
 		goto out;
 out:
@@ -741,7 +779,8 @@ out:
  * gcm_calibrate_argyll_device_copy:
  **/
 static gboolean
-gcm_calibrate_argyll_device_copy (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+gcm_calibrate_argyll_device_copy (GcmCalibrateArgyll *calibrate_argyll,
+				  GError **error)
 {
 	gboolean ret;
 	gchar *device = NULL;
@@ -773,10 +812,13 @@ gcm_calibrate_argyll_device_copy (GcmCalibrateArgyll *calibrate_argyll, GError *
 
 	/* TRANSLATORS: title, a profile is a ICC file */
 	gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-				 _("Copying files"));
+				 _("Copying files"),
+				 GCM_CALIBRATE_UI_STATUS);
+
 	/* TRANSLATORS: dialog message */
 	gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   _("Copying source image, chart data and CIE reference values."));
+				   _("Copying source image, chart data and CIE reference values."),
+				   GCM_CALIBRATE_UI_STATUS);
 
 	/* build filenames */
 	filename = g_strdup_printf ("%s.tif", basename);
@@ -828,7 +870,8 @@ out:
  * gcm_calibrate_argyll_device_measure:
  **/
 static gboolean
-gcm_calibrate_argyll_device_measure (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+gcm_calibrate_argyll_device_measure (GcmCalibrateArgyll *calibrate_argyll,
+				     GError **error)
 {
 	gboolean ret = TRUE;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
@@ -845,10 +888,13 @@ gcm_calibrate_argyll_device_measure (GcmCalibrateArgyll *calibrate_argyll, GErro
 
 	/* TRANSLATORS: title, drawing means painting to the screen */
 	gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-				 _("Measuring the patches"));
+				 _("Measuring the patches"),
+				 GCM_CALIBRATE_UI_STATUS);
+
 	/* TRANSLATORS: dialog message */
 	gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   _("Detecting the reference patches and measuring them."));
+				   _("Detecting the reference patches and measuring them."),
+				   GCM_CALIBRATE_UI_STATUS);
 
 	/* get correct name of the command */
 	command = gcm_calibrate_argyll_get_tool_filename ("scanin", error);
@@ -918,7 +964,8 @@ out:
  * gcm_calibrate_argyll_device_generate_profile:
  **/
 static gboolean
-gcm_calibrate_argyll_device_generate_profile (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+gcm_calibrate_argyll_device_generate_profile (GcmCalibrateArgyll *calibrate_argyll,
+					      GError **error)
 {
 	gboolean ret = TRUE;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
@@ -951,10 +998,13 @@ gcm_calibrate_argyll_device_generate_profile (GcmCalibrateArgyll *calibrate_argy
 
 	/* TRANSLATORS: title, a profile is a ICC file */
 	gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-				 _("Generating the profile"));
+				 _("Generating the profile"),
+				 GCM_CALIBRATE_UI_STATUS);
+
 	/* TRANSLATORS: dialog message */
 	gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   _("Generating the ICC color profile that can be used with this device."));
+				   _("Generating the ICC color profile that can be used with this device."),
+				   GCM_CALIBRATE_UI_STATUS);
 
 	/* argument array */
 	array = g_ptr_array_new_with_free_func (g_free);
@@ -1026,7 +1076,8 @@ out:
  * gcm_calibrate_argyll_set_filename_result:
  **/
 static gboolean
-gcm_calibrate_argyll_set_filename_result (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+gcm_calibrate_argyll_set_filename_result (GcmCalibrateArgyll *calibrate_argyll,
+					  GError **error)
 {
 	gchar *filename = NULL;
 	gboolean ret = TRUE;
@@ -1076,7 +1127,8 @@ out:
  * gcm_calibrate_argyll_remove_temp_files:
  **/
 static gboolean
-gcm_calibrate_argyll_remove_temp_files (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+gcm_calibrate_argyll_remove_temp_files (GcmCalibrateArgyll *calibrate_argyll,
+					GError **error)
 {
 	gchar *filename_tmp;
 	guint i;
@@ -1127,7 +1179,11 @@ gcm_calibrate_argyll_remove_temp_files (GcmCalibrateArgyll *calibrate_argyll, GE
  * gcm_calibrate_argyll_display:
  **/
 static gboolean
-gcm_calibrate_argyll_display (GcmCalibrate *calibrate, CdDevice *device, CdSensor *sensor, GtkWindow *window, GError **error)
+gcm_calibrate_argyll_display (GcmCalibrate *calibrate,
+			      CdDevice *device,
+			      CdSensor *sensor,
+			      GtkWindow *window,
+			      GError **error)
 {
 	GcmCalibrateArgyll *calibrate_argyll = GCM_CALIBRATE_ARGYLL(calibrate);
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
@@ -1213,7 +1269,6 @@ gcm_calibrate_argyll_interaction (GcmCalibrate *calibrate, GtkResponseType respo
 #ifdef HAVE_VTE
 			vte_terminal_feed_child (VTE_TERMINAL(priv->terminal), priv->argyllcms_ok, 1);
 #endif
-			gcm_calibrate_pop (calibrate);
 			priv->state = GCM_CALIBRATE_ARGYLL_STATE_RUNNING;
 		}
 
@@ -1222,7 +1277,6 @@ gcm_calibrate_argyll_interaction (GcmCalibrate *calibrate, GtkResponseType respo
 			g_main_loop_quit (priv->loop);
 			priv->state = GCM_CALIBRATE_ARGYLL_STATE_RUNNING;
 		}
-
 		goto out;
 	}
 
@@ -1283,10 +1337,13 @@ gcm_calibrate_argyll_get_sensor_target (GcmCalibrateArgyll *calibrate_argyll)
 }
 
 /**
- * gcm_calibrate_argyll_display_generate_targets:
+ * gcm_calibrate_argyll_printer_generate_targets:
  **/
 static gboolean
-gcm_calibrate_argyll_display_generate_targets (GcmCalibrateArgyll *calibrate_argyll, gdouble width, gdouble height, GError **error)
+gcm_calibrate_argyll_printer_generate_targets (GcmCalibrateArgyll *calibrate_argyll,
+					       gdouble width,
+					       gdouble height,
+					       GError **error)
 {
 	gboolean ret = TRUE;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
@@ -1311,11 +1368,13 @@ gcm_calibrate_argyll_display_generate_targets (GcmCalibrateArgyll *calibrate_arg
 
 	/* TRANSLATORS: title, patches are specific colors used in calibration */
 	gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-				 _("Printing patches"));
+				 _("Printing patches"),
+				 GCM_CALIBRATE_UI_STATUS);
 
 	/* TRANSLATORS: dialog message */
 	gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   _("Rendering the patches for the selected paper and ink."));
+				   _("Rendering the patches for the selected paper and ink."),
+				   GCM_CALIBRATE_UI_STATUS);
 
 	/* argument array */
 	array = g_ptr_array_new_with_free_func (g_free);
@@ -1386,11 +1445,37 @@ out:
 	return ret;
 }
 
+/**
+ * gcm_calibrate_argyll_get_enabled:
+ **/
+static gboolean
+gcm_calibrate_argyll_get_enabled (GcmCalibrate *calibrate)
+{
+	gboolean ret = TRUE;
+	gchar *command;
+	GError *error = NULL;
+
+	/* get correct name of the command */
+	command = gcm_calibrate_argyll_get_tool_filename ("dispcal", &error);
+	if (command == NULL) {
+		g_debug ("Failed to find dispcal: %s", error->message);
+		g_error_free (error);
+		ret = FALSE;
+		goto out;
+	}
+out:
+	g_free (command);
+	return ret;
+}
+
 /**
  * gcm_calibrate_argyll_render_cb:
  **/
 static GPtrArray *
-gcm_calibrate_argyll_render_cb (GcmPrint *print, GtkPageSetup *page_setup, GcmCalibrate *calibrate, GError **error)
+gcm_calibrate_argyll_render_cb (GcmPrint *print,
+				GtkPageSetup *page_setup,
+				GcmCalibrate *calibrate,
+				GError **error)
 {
 	gboolean ret = TRUE;
 	GPtrArray *array = NULL;
@@ -1413,7 +1498,10 @@ gcm_calibrate_argyll_render_cb (GcmPrint *print, GtkPageSetup *page_setup, GcmCa
 	height = gtk_paper_size_get_height (paper_size, GTK_UNIT_MM);
 
 	/* generate the tiff files */
-	ret = gcm_calibrate_argyll_display_generate_targets (GCM_CALIBRATE_ARGYLL (calibrate), width, height, error);
+	ret = gcm_calibrate_argyll_printer_generate_targets (GCM_CALIBRATE_ARGYLL (calibrate),
+							     width,
+							     height,
+							     error);
 	if (!ret)
 		goto out;
 
@@ -1445,7 +1533,9 @@ out:
  * gcm_calibrate_argyll_set_device_from_ti2:
  **/
 static gboolean
-gcm_calibrate_argyll_set_device_from_ti2 (GcmCalibrate *calibrate, const gchar *filename, GError **error)
+gcm_calibrate_argyll_set_device_from_ti2 (GcmCalibrate *calibrate,
+					  const gchar *filename,
+					  GError **error)
 {
 	gboolean ret;
 	gchar *contents = NULL;
@@ -1523,7 +1613,8 @@ out:
  * gcm_calibrate_argyll_printer_convert_jpeg:
  **/
 static gboolean
-gcm_calibrate_argyll_printer_convert_jpeg (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+gcm_calibrate_argyll_printer_convert_jpeg (GcmCalibrateArgyll *calibrate_argyll,
+					   GError **error)
 {
 	GDir *dir;
 	const gchar *filename;
@@ -1563,7 +1654,13 @@ gcm_calibrate_argyll_printer_convert_jpeg (GcmCalibrateArgyll *calibrate_argyll,
 			}
 
 			/* try to save new file */
-			ret = gdk_pixbuf_save (pixbuf, filename_jpg, "jpeg", error, "quality", "100", NULL);
+			ret = gdk_pixbuf_save (pixbuf,
+					       filename_jpg,
+					       "jpeg",
+					       error,
+					       "quality",
+					       "100",
+					       NULL);
 			if (!ret)
 				goto out;
 			g_object_unref (pixbuf);
@@ -1582,7 +1679,11 @@ out:
  * gcm_calibrate_argyll_printer:
  **/
 static gboolean
-gcm_calibrate_argyll_printer (GcmCalibrate *calibrate, CdDevice *device, CdSensor *sensor, GtkWindow *window, GError **error)
+gcm_calibrate_argyll_printer (GcmCalibrate *calibrate,
+			      CdDevice *device,
+			      CdSensor *sensor,
+			      GtkWindow *window,
+			      GError **error)
 {
 	gboolean ret;
 	gchar *cmdline = NULL;
@@ -1604,7 +1705,11 @@ gcm_calibrate_argyll_printer (GcmCalibrate *calibrate, CdDevice *device, CdSenso
 
 	/* print */
 	if (print_kind == GCM_CALIBRATE_PRINT_KIND_LOCAL) {
-		ret = gcm_print_with_render_callback (priv->print, window, (GcmPrintRenderCb) gcm_calibrate_argyll_render_cb, calibrate, error);
+		ret = gcm_print_with_render_callback (priv->print,
+						      window,
+						      (GcmPrintRenderCb) gcm_calibrate_argyll_render_cb,
+						      calibrate,
+						      error);
 		if (!ret)
 			goto out;
 	}
@@ -1618,12 +1723,16 @@ gcm_calibrate_argyll_printer (GcmCalibrate *calibrate, CdDevice *device, CdSenso
 		height = gtk_paper_size_get_height (paper_size, GTK_UNIT_MM);
 
 		/* generate the tiff files */
-		ret = gcm_calibrate_argyll_display_generate_targets (GCM_CALIBRATE_ARGYLL (calibrate), width, height, error);
+		ret = gcm_calibrate_argyll_printer_generate_targets (GCM_CALIBRATE_ARGYLL (calibrate),
+								     width,
+								     height,
+								     error);
 		if (!ret)
 			goto out;
 
 		/* convert to jpegs */
-		ret = gcm_calibrate_argyll_printer_convert_jpeg (GCM_CALIBRATE_ARGYLL (calibrate), error);
+		ret = gcm_calibrate_argyll_printer_convert_jpeg (GCM_CALIBRATE_ARGYLL (calibrate),
+								 error);
 		if (!ret)
 			goto out;
 
@@ -1637,11 +1746,13 @@ gcm_calibrate_argyll_printer (GcmCalibrate *calibrate, CdDevice *device, CdSenso
 	if (print_kind == GCM_CALIBRATE_PRINT_KIND_LOCAL) {
 		/* TRANSLATORS: title, patches are specific colours used in calibration */
 		gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-					 _("Wait for the ink to dry"));
+					 _("Wait for the ink to dry"),
+					 GCM_CALIBRATE_UI_STATUS);
 
 		/* TRANSLATORS: dialog message */
 		gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-					   _("Please wait a few minutes for the ink to dry. Profiling damp ink will produce a poor profile and may damage your color measuring instrument."));
+					   _("Please wait a few minutes for the ink to dry. Profiling damp ink will produce a poor profile and may damage your color measuring instrument."),
+					   GCM_CALIBRATE_UI_STATUS);
 
 	}
 
@@ -1676,20 +1787,23 @@ gcm_calibrate_argyll_printer (GcmCalibrate *calibrate, CdDevice *device, CdSenso
 	gcm_calibrate_set_progress (calibrate, 80);
 
 	/* step 4 */
-	ret = gcm_calibrate_argyll_device_generate_profile (calibrate_argyll, error);
+	ret = gcm_calibrate_argyll_device_generate_profile (calibrate_argyll,
+							    error);
 	if (!ret)
 		goto out;
 
 	/* only delete state if we are doing a local printer */
 	if (print_kind == GCM_CALIBRATE_PRINT_KIND_LOCAL) {
 		/* step 5 */
-		ret = gcm_calibrate_argyll_remove_temp_files (calibrate_argyll, error);
+		ret = gcm_calibrate_argyll_remove_temp_files (calibrate_argyll,
+							      error);
 		if (!ret)
 			goto out;
 	}
 
 	/* step 6 */
-	ret = gcm_calibrate_argyll_set_filename_result (calibrate_argyll, error);
+	ret = gcm_calibrate_argyll_set_filename_result (calibrate_argyll,
+							error);
 	if (!ret)
 		goto out;
 out:
@@ -1740,7 +1854,8 @@ out:
  * gcm_calibrate_argyll_check_and_remove_alpha:
  **/
 static gboolean
-gcm_calibrate_argyll_check_and_remove_alpha (GcmCalibrateArgyll *calibrate_argyll, GError **error)
+gcm_calibrate_argyll_check_and_remove_alpha (GcmCalibrateArgyll *calibrate_argyll,
+					     GError **error)
 {
 	const gchar *working_path;
 	gboolean ret = TRUE;
@@ -1802,18 +1917,24 @@ out:
  * gcm_calibrate_argyll_device:
  **/
 static gboolean
-gcm_calibrate_argyll_device (GcmCalibrate *calibrate, CdDevice *device, CdSensor *sensor, GtkWindow *window, GError **error)
+gcm_calibrate_argyll_device (GcmCalibrate *calibrate,
+			     CdDevice *device,
+			     CdSensor *sensor,
+			     GtkWindow *window,
+			     GError **error)
 {
 	GcmCalibrateArgyll *calibrate_argyll = GCM_CALIBRATE_ARGYLL(calibrate);
 	gboolean ret;
 
 	/* TRANSLATORS: title, instrument refers to a calibration device */
 	gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-				 _("Set up instrument"));
+				 _("Set up instrument"),
+				 GCM_CALIBRATE_UI_STATUS);
 
 	/* TRANSLATORS: dialog message */
 	gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   _("Setting up the instrument for use…"));
+				   _("Setting up the instrument for use…"),
+				   GCM_CALIBRATE_UI_STATUS);
 
 
 	/* step 1 */
@@ -1862,15 +1983,22 @@ out:
 }
 
 #ifdef HAVE_VTE
+
+static void gcm_calibrate_argyll_flush_vte (GcmCalibrateArgyll *calibrate_argyll);
+
 /**
  * gcm_calibrate_argyll_exit_cb:
  **/
 static void
-gcm_calibrate_argyll_exit_cb (VteTerminal *terminal, GcmCalibrateArgyll *calibrate_argyll)
+gcm_calibrate_argyll_exit_cb (VteTerminal *terminal,
+			      GcmCalibrateArgyll *calibrate_argyll)
 {
 	gint exit_status;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
 
+	/* flush the VTE output */
+	gcm_calibrate_argyll_flush_vte (calibrate_argyll);
+
 	/* get the child exit status */
 	exit_status = vte_terminal_get_child_exit_status (terminal);
 	g_debug ("child exit-status is %i", exit_status);
@@ -1979,7 +2107,8 @@ gcm_calibrate_argyll_interaction_surface (GcmCalibrateArgyll *calibrate_argyll)
  * Return value: if FALSE then abort processing input
  **/
 static gboolean
-gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, const gchar *line)
+gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll,
+					 const gchar *line)
 {
 	gchar *title_str = NULL;
 	GString *string = NULL;
@@ -2016,11 +2145,13 @@ gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, c
 	if (g_strstr_len (line, -1, "Measurement misread") != NULL) {
 		/* TRANSLATORS: title, the calibration failed */
 		gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-					 _("Calibration error"));
+					 _("Calibration error"),
+					 GCM_CALIBRATE_UI_ERROR);
 
 		/* TRANSLATORS: message, the sample was not read correctly */
 		gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-					   _("The sample could not be read at this time."));
+					   _("The sample could not be read at this time."),
+					   GCM_CALIBRATE_UI_ERROR);
 
 
 		/* set state */
@@ -2080,28 +2211,34 @@ gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, c
 
 		/* TRANSLATORS: title, the calibration failed */
 		gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-					 _("Calibration error"));
+					 _("Calibration error"),
+					 GCM_CALIBRATE_UI_ERROR);
 
 		if (g_strstr_len (line, -1, "No PLD firmware pattern is available") != NULL) {
 			/* TRANSLATORS: message, no firmware is available */
 			gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   _("No firmware is installed for this instrument."));
+						   _("No firmware is installed for this instrument."),
+						   GCM_CALIBRATE_UI_ERROR);
 		} else if (g_strstr_len (line, -1, "Pattern match wasn't good enough") != NULL) {
 			/* TRANSLATORS: message, the image wasn't good enough */
 			gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   _("The pattern match wasn't good enough. Ensure you have the correct type of target selected."));
+						   _("The pattern match wasn't good enough. Ensure you have the correct type of target selected."),
+						   GCM_CALIBRATE_UI_ERROR);
 		} else if (g_strstr_len (line, -1, "Aprox. fwd matrix unexpectedly singular") != NULL ||
 			   g_strstr_len (line, -1, "Inverting aprox. fwd matrix failed") != NULL) {
 			/* TRANSLATORS: message, the sensor got no readings */
 			gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   _("The measuring instrument got no valid readings. Please ensure the aperture is fully open."));
+						   _("The measuring instrument got no valid readings. Please ensure the aperture is fully open."),
+						   GCM_CALIBRATE_UI_ERROR);
 		} else if (g_strstr_len (line, -1, "Device or resource busy") != NULL) {
 			/* TRANSLATORS: message, the colorimeter has got confused */
 			gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   _("The measuring instrument is busy and is not starting up. Please remove the USB plug and re-insert before trying to use this device."));
+						   _("The measuring instrument is busy and is not starting up. Please remove the USB plug and re-insert before trying to use this device."),
+						   GCM_CALIBRATE_UI_ERROR);
 		} else {
 			gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-				   found + 8);
+						   found + 8,
+						   GCM_CALIBRATE_UI_ERROR);
 		}
 
 
@@ -2134,11 +2271,13 @@ gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, c
 	if (g_str_has_prefix (line, "Strip read failed due to misread")) {
 		/* TRANSLATORS: dialog title */
 		gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-					 _("Reading target"));
+					 _("Reading target"),
+					 GCM_CALIBRATE_UI_ERROR);
 
 		/* TRANSLATORS: message, no firmware is available */
 		gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-					   _("Failed to read the strip correctly."));
+					   _("Failed to read the strip correctly."),
+					   GCM_CALIBRATE_UI_ERROR);
 
 		/* set state */
 		priv->argyllcms_ok = "\n";
@@ -2154,6 +2293,32 @@ gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, c
 		goto out;
 	}
 
+	/* reading sample */
+	if (g_str_has_prefix (line, "Sample read failed due to misread")) {
+		/* TRANSLATORS: dialog title */
+		gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
+					 _("Reading sample"),
+					 GCM_CALIBRATE_UI_ERROR);
+
+		/* TRANSLATORS: message, no firmware is available */
+		gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
+					   _("Failed to read the color sample correctly."),
+					   GCM_CALIBRATE_UI_ERROR);
+
+		/* set state */
+		priv->argyllcms_ok = "\n";
+		priv->state = GCM_CALIBRATE_ARGYLL_STATE_WAITING_FOR_STDIN;
+		gcm_calibrate_interaction_required (GCM_CALIBRATE (calibrate_argyll), _("Retry"));
+
+		/* play sound from the naming spec */
+		ca_context_play (ca_gtk_context_get (), 0,
+				 CA_PROP_EVENT_ID, "dialog-warning",
+				 /* TRANSLATORS: this is the application name for libcanberra */
+				 CA_PROP_APPLICATION_NAME, _("GNOME Color Manager"),
+				 CA_PROP_EVENT_DESCRIPTION, "failed to read sample", NULL);
+		goto out;
+	}
+
 	/* reading strip */
 	if (g_str_has_prefix (line, "(Warning) Seem to have read strip pass ")) {
 
@@ -2174,8 +2339,6 @@ gcm_calibrate_argyll_process_output_cmd (GcmCalibrateArgyll *calibrate_argyll, c
 		g_string_append (string, _("If you've really measured the right one, it's okay, it could just be unusual paper."));
 		g_string_append (string, "\n\n");
 
-g_debug ("title=%s, message=%s", title_str, string->str);
-
 		/* push new messages into the UI */
 		gcm_calibrate_set_image (GCM_CALIBRATE (calibrate_argyll), "scan-target-bad.svg");
 
@@ -2198,11 +2361,13 @@ g_debug ("title=%s, message=%s", title_str, string->str);
 
 		/* TRANSLATORS: title, the calibration failed */
 		gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-					 _("Device Error"));
+					 _("Device Error"),
+					 GCM_CALIBRATE_UI_ERROR);
 
 		/* TRANSLATORS: message, the sample was not read correctly */
 		gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-					   _("The device could not measure the color spot correctly."));
+					   _("The device could not measure the color spot correctly."),
+					   GCM_CALIBRATE_UI_ERROR);
 
 		/* set state */
 		priv->argyllcms_ok = " ";
@@ -2239,8 +2404,26 @@ g_debug ("title=%s, message=%s", title_str, string->str);
 		goto out;
 	}
 
+	/* update the percentage bar */
+	if (g_str_has_prefix (line, "patch ")) {
+		guint total;
+		guint current;
+		split = g_strsplit (line, " ", -1);
+		if (g_strv_length (split) != 4) {
+			g_warning ("invalid update format: %s", line);
+			goto out;
+		}
+		current = atoi (split[1]);
+		total = atoi (split[3]);
+		if (total > 0) {
+			gcm_calibrate_set_progress (GCM_CALIBRATE (calibrate_argyll),
+						    current * 100 / total);
+		}
+		goto out;
+	}
+
 	/* report a warning so friendly people report bugs */
-	g_warning ("VTE: could not screenscrape: %s", line);
+	g_warning ("VTE: could not screenscrape: '%s'", line);
 out:
 	g_free (title_str);
 	g_strfreev (split);
@@ -2255,17 +2438,20 @@ out:
  * gcm_calibrate_argyll_selection_func_cb:
  **/
 static gboolean
-gcm_calibrate_argyll_selection_func_cb (VteTerminal *terminal, glong column, glong row, gpointer data)
+gcm_calibrate_argyll_selection_func_cb (VteTerminal *terminal,
+					glong column,
+					glong row,
+					gpointer data)
 {
 	/* just select everything */
 	return TRUE;
 }
 
 /**
- * gcm_calibrate_argyll_cursor_moved_cb:
+ * gcm_calibrate_argyll_flush_vte:
  **/
 static void
-gcm_calibrate_argyll_cursor_moved_cb (VteTerminal *terminal, GcmCalibrateArgyll *calibrate_argyll)
+gcm_calibrate_argyll_flush_vte (GcmCalibrateArgyll *calibrate_argyll)
 {
 	gchar *output;
 	gchar **split;
@@ -2274,6 +2460,7 @@ gcm_calibrate_argyll_cursor_moved_cb (VteTerminal *terminal, GcmCalibrateArgyll
 	glong col;
 	gboolean ret;
 	GcmCalibrateArgyllPrivate *priv = calibrate_argyll->priv;
+	VteTerminal *terminal = VTE_TERMINAL (priv->terminal);
 
 	/* select the text we've got since last time */
 	vte_terminal_get_cursor_position (terminal, &col, &row);
@@ -2301,17 +2488,31 @@ gcm_calibrate_argyll_cursor_moved_cb (VteTerminal *terminal, GcmCalibrateArgyll
 	g_free (output);
 	g_strfreev (split);
 }
+
+/**
+ * gcm_calibrate_argyll_cursor_moved_cb:
+ **/
+static void
+gcm_calibrate_argyll_cursor_moved_cb (VteTerminal *terminal,
+				      GcmCalibrateArgyll *calibrate_argyll)
+{
+	gcm_calibrate_argyll_flush_vte (calibrate_argyll);
+}
+
 #endif
 
 /**
  * gcm_calibrate_argyll_status_changed_cb:
  **/
 static void
-gcm_calibrate_argyll_status_changed_cb (GcmPrint *print, GtkPrintStatus status, GcmCalibrateArgyll *calibrate_argyll)
+gcm_calibrate_argyll_status_changed_cb (GcmPrint *print,
+					GtkPrintStatus status,
+					GcmCalibrateArgyll *calibrate_argyll)
 {
 	/* TRANSLATORS: title, printing reference files to media */
 	gcm_calibrate_set_title (GCM_CALIBRATE (calibrate_argyll),
-				 _("Printing"));
+				 _("Printing"),
+				 GCM_CALIBRATE_UI_STATUS);
 
 	switch (status) {
 	case GTK_PRINT_STATUS_INITIAL:
@@ -2319,29 +2520,34 @@ gcm_calibrate_argyll_status_changed_cb (GcmPrint *print, GtkPrintStatus status,
 	case GTK_PRINT_STATUS_GENERATING_DATA:
 		/* TRANSLATORS: dialog message */
 		gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-					   _("Preparing the data for the printer."));
+					   _("Preparing the data for the printer."),
+					   GCM_CALIBRATE_UI_STATUS);
 		break;
 	case GTK_PRINT_STATUS_SENDING_DATA:
 	case GTK_PRINT_STATUS_PENDING:
 	case GTK_PRINT_STATUS_PENDING_ISSUE:
 		/* TRANSLATORS: dialog message */
 		gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-					   _("Sending the targets to the printer."));
+					   _("Sending the targets to the printer."),
+					   GCM_CALIBRATE_UI_STATUS);
 		break;
 	case GTK_PRINT_STATUS_PRINTING:
 		/* TRANSLATORS: dialog message */
 		gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-					   _("Printing the targets..."));
+					   _("Printing the targets..."),
+					   GCM_CALIBRATE_UI_STATUS);
 		break;
 	case GTK_PRINT_STATUS_FINISHED:
 		/* TRANSLATORS: dialog message */
 		gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-					   _("The printing has finished."));
+					   _("The printing has finished."),
+					   GCM_CALIBRATE_UI_STATUS);
 		break;
 	case GTK_PRINT_STATUS_FINISHED_ABORTED:
 		/* TRANSLATORS: dialog message */
 		gcm_calibrate_set_message (GCM_CALIBRATE (calibrate_argyll),
-					   _("The print was aborted."));
+					   _("The print was aborted."),
+					   GCM_CALIBRATE_UI_STATUS);
 		break;
 	default:
 		break;
@@ -2362,6 +2568,7 @@ gcm_calibrate_argyll_class_init (GcmCalibrateArgyllClass *klass)
 	parent_class->calibrate_display = gcm_calibrate_argyll_display;
 	parent_class->calibrate_device = gcm_calibrate_argyll_device;
 	parent_class->calibrate_printer = gcm_calibrate_argyll_printer;
+	parent_class->get_enabled = gcm_calibrate_argyll_get_enabled;
 	parent_class->interaction = gcm_calibrate_argyll_interaction;
 
 	g_type_class_add_private (klass, sizeof (GcmCalibrateArgyllPrivate));
